/*
 * Copyright 2007-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file sigmod/MapWarp.cpp
 */

// Header include
#include "MapWarp.h"

// Sigmod includes
#include "Game.h"
#include "Macros.h"
#include "Map.h"

using namespace Sigcore;
using namespace Sigmod;

const QStringList MapWarp::TypeStr = QStringList() << "Door" << "Warp" << "Hole" << "Boundary";

MapWarp::MapWarp(const MapWarp& warp) :
        Object(warp.parent(), warp.id())
{
    *this = warp;
}

MapWarp::MapWarp(const Map* parent, const int id) :
        Object(parent, id),
        m_name(""),
        m_area(0, 0, 32, 32),
        m_type(Door),
        m_toMap(INT_MAX),
        m_toWarp(INT_MAX),
        m_script("", "")
{
}

MapWarp::MapWarp(const MapWarp& warp, const Map* parent, const int id) :
        Object(parent, id)
{
    *this = warp;
}

MapWarp::MapWarp(const QDomElement& xml, const Map* parent, const int id) :
        Object(parent, id)
{
    LOAD_ID();
    load(xml);
}

void MapWarp::validate()
{
    TEST_BEGIN();
    if (m_name.isEmpty())
        emit(error("Name is empty"));
    TEST(area);
    TEST(toMap);
    TEST(toWarp);
    TEST_END();
}

void MapWarp::load(const QDomElement& xml)
{
    LOAD_BEGIN();
    LOAD(name);
    LOAD(area);
    LOAD_ENUM(type, Type);
    LOAD(toMap);
    LOAD(toWarp);
    LOAD(script);
}

QDomElement MapWarp::save() const
{
    SAVE_CREATE();
    SAVE(name);
    SAVE(area);
    SAVE_ENUM(type, Type);
    SAVE(toMap);
    SAVE(toWarp);
    SAVE(script);
    return xml;
}

SETTER(MapWarp, QString&, Name, name)
SETTER(MapWarp, QRect&, Area, area)
SETTER(MapWarp, Type, Type, type)
SETTER(MapWarp, int, ToMap, toMap)
SETTER(MapWarp, int, ToWarp, toWarp)
SETTER(MapWarp, Script&, Script, script)

GETTER(MapWarp, QString, name)
GETTER(MapWarp, QRect, area)
GETTER(MapWarp, MapWarp::Type, type)
GETTER(MapWarp, int, toMap)
GETTER(MapWarp, int, toWarp)
GETTER(MapWarp, Script, script)

CHECK(MapWarp, QString&, name)
CHECK_BEGIN(MapWarp, QRect&, area)
    const Map* map = qobject_cast<const Map*>(parent());
    TBOUNDS_MOD(area_x, 0, map->width() - 1, area.x());
    TBOUNDS_MOD(area_y, 0, map->height() - 1, area.y());
    TBOUNDS_MOD(area_width, 1, map->width() - area.x(), area.width());
    TBOUNDS_MOD(area_height, 1, map->height() - area.y(), area.height());
CHECK_END()
CHECK(MapWarp, Type, type)
CHECK_INDEX(MapWarp, int, toMap, game(), map)
CHECK_BEGIN(MapWarp, int, toWarp)
    const Map* map = game()->mapById(m_toMap);
    if (!map)
    {
        EBOUNDS_IDX(m_toMap);
        return false;
    }
    IBOUNDS(toWarp, map, warp);
CHECK_END()
CHECK(MapWarp, Script&, script)

MapWarp& MapWarp::operator=(const MapWarp& rhs)
{
    if (this == &rhs)
        return *this;
    COPY(name);
    COPY(area);
    COPY(type);
    COPY(toMap);
    COPY(toWarp);
    COPY(script);
    return *this;
}
